(* $Id: walk_output.ml,v 1.3 2012/05/22 15:11:17 fclement Exp $ *)

(* Generate a printing program for values. *)
module Output : Action_spec.Make =
  functor (Io : Walk_spec.Out) ->

struct

  let ppf = Io.buffer
  ;;

  let print_seq () =
    Format.fprintf ppf ";@ "
  ;;

  let meta_print_string s =
    Format.fprintf ppf "@[<1>Format.fprintf ppf@ \"%s\"@]" (String.escaped s)
  ;;

  let quoted_string s = "\"" ^ s ^ "\""
  ;;

  let sum_tag lab = meta_print_string lab
  ;;

  let begin_sum_inj lab =
    Format.fprintf ppf
      "@[Format.fprintf ppf@ %s@ %s;@ "
      (quoted_string "@[<1>(%s@ ") (quoted_string lab)
  ;;
  let end_sum_inj _s =
    print_seq ();
    meta_print_string ")@]";
    Format.fprintf ppf "@]"
  ;;

  let begin_tuple () =
    meta_print_string "@[<1>("; print_seq ()
  ;;
    let begin_tuple_elem () = ()
    ;;
    let end_tuple_elem () = ()
    ;;
    let separator_tuple () =
      print_seq ();
      meta_print_string ",@ ";
      print_seq ()
    ;;
  let end_tuple () =
    print_seq (); meta_print_string ")@]"
  ;;

  let begin_record_expr () =
    meta_print_string "@[<1>{"; print_seq ()
  ;;
  let end_record_expr () =
    meta_print_string "@,}@]"
  ;;

  let begin_record_field _lab = ()
  ;;
  let end_record_field _lab =
    print_seq (); meta_print_string ";"; print_seq ()
  ;;

  let begin_application () =
    Format.fprintf ppf "@[<1>"
  ;;
    let separator_argument () = Format.fprintf ppf "@ "
    ;;
    let begin_argument () = ()
    ;;
    let end_argument () = ()
    ;;
  let end_application () =
    Format.fprintf ppf "@]"
  ;;

  let external_fun = function
    | "output_bool" -> Format.fprintf ppf "Lib_output.output_bool"
    | "output_string" -> Format.fprintf ppf "Lib_output.output_quoted_string"
    | "output_int" -> Format.fprintf ppf "Lib_output.output_quoted_int"
    | "output_float" -> Format.fprintf ppf "Lib_output.output_quoted_float"
    | "output_char" -> Format.fprintf ppf "Lib_output.output_quoted_char"
    | "output_list" -> Format.fprintf ppf "Lib_output.output_List"
    | "output_array" -> Format.fprintf ppf "Lib_output.output_Array"
    | "output_option" -> Format.fprintf ppf "Lib_output.output_Option"
    | "output_fun" -> Format.fprintf ppf "Lib_output.output_fun"
    | "output_poly" -> Format.fprintf ppf "Lib_output.output_poly"
    | s -> Format.fprintf ppf "%s" s
  ;;

  let begin_paren () =
    meta_print_string "@[<1>("; print_seq ()
  ;;

  let end_paren () =
    print_seq ();
    meta_print_string "@,)@]"
  ;;

end;;

module Gen : Walk_spec.Make = Walk_gen.Gen;;

(* Generates a value printing program. *)
module Outputer =
 functor (Io : Walk_spec.Out) ->
 Walker.Make (Gen) (Output) (Io)
;;
