(***********************************************************************)
(*                                                                     *)
(*                              OCamlGen                               *)
(*                                                                     *)
(*                       OCaml module Generator                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 1997-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: walk_print_xml.ml,v 1.10 2014/01/31 09:51:53 fclement Exp $ *)

(* Generate a printing program for values in XML syntax. *)

module Print_xml : Action_spec.Make =
  functor (Io : Walk_spec.Out) ->

  struct

    let ppf = Io.buffer;;

    let print_seq () = Format.fprintf ppf ";@ ";;

    let meta_print_string s =
      Format.fprintf ppf
        "@[<1>Format.fprintf ppf@ \"%%,%s\"@]"
        (String.escaped s)
    ;;

    let quoted_string s = "\"" ^ s ^ "\"";;

    let sum_tag lab =
      Format.fprintf ppf
        "@[Format.fprintf ppf@ %s@ %s@]"
        (quoted_string "%s")
        (quoted_string ("<" ^ lab ^ "/>"))
    ;;

    let begin_sum_inj lab =
      Format.fprintf ppf
        "@[Format.fprintf ppf@ %s@ %s;@ "
        (quoted_string "@[<1>%s@,")
        (quoted_string ("<" ^ lab ^ ">"))
    ;;
    let end_sum_inj lab =
      print_seq ();
      meta_print_string ("</" ^ lab ^ ">"); print_seq ();
      meta_print_string "@]";
      Format.fprintf ppf "@]"
    ;;

    let begin_tuple () =
      meta_print_string "@[<1>";
      print_seq ()
    ;;
      let begin_tuple_elem () = ();;
      let end_tuple_elem () = ();;
      let separator_tuple () =
        print_seq ();
        meta_print_string "@ ";
        print_seq ()
      ;;
    let end_tuple () =
      print_seq ();
      meta_print_string "@]"
    ;;

    let begin_record_expr () =
      meta_print_string "@[<1>{@,"; print_seq ()
    ;;
    let end_record_expr () = meta_print_string "@,}@]";;

    let begin_record_field _lab = ();;
    let end_record_field _lab =
      print_seq (); meta_print_string ";"; print_seq ()
    ;;

    let begin_application () = Format.fprintf ppf "%,@[<1>";;
      let separator_argument () = Format.fprintf ppf "%,@ ";;
      let begin_argument () = ();;
      let end_argument () = ();;
    let end_application () = Format.fprintf ppf "%,@]";;

    let external_fun = function
      | "print_xml_unit" ->
        Format.fprintf ppf "Lib_print_xml.print_xml_unit"
      | "print_xml_bool" ->
        Format.fprintf ppf "Lib_print_xml.print_xml_bool"
      | "print_xml_string" ->
        Format.fprintf ppf "Lib_print_xml.print_xml_quoted_string"
      | "print_xml_int" ->
        Format.fprintf ppf "Lib_print_xml.print_xml_quoted_int"
      | "print_xml_nativeint" ->
        Format.fprintf ppf "Lib_print_xml.print_xml_quoted_nativeint"
      | "print_xml_int32" ->
        Format.fprintf ppf "Lib_print_xml.print_xml_quoted_int32"
      | "print_xml_int64" ->
        Format.fprintf ppf "Lib_print_xml.print_xml_quoted_int64"
      | "print_xml_float" ->
        Format.fprintf ppf "Lib_print_xml.print_xml_quoted_float"
      | "print_xml_char" ->
        Format.fprintf ppf "Lib_print_xml.print_xml_quoted_char"
      | "print_xml_list" ->
        Format.fprintf ppf "Lib_print_xml.print_xml_List"
      | "print_xml_array" ->
        Format.fprintf ppf "Lib_print_xml.print_xml_Array"
      | "print_xml_option" ->
        Format.fprintf ppf "Lib_print_xml.print_xml_Option"
      | "print_xml_ref" ->
        Format.fprintf ppf "Lib_print_xml.print_xml_Ref"
      | "print_xml_fun" ->
        Format.fprintf ppf "Lib_print_xml.print_xml_fun"
      | "print_xml_poly" ->
        Format.fprintf ppf "Lib_print_xml.print_xml_poly"
      | "print_xml_abstract" ->
        Format.fprintf ppf "Lib_print_xml.print_xml_abstract"
      | "print_xml_external" ->
        Format.fprintf ppf "Lib_print_xml.print_xml_external"
      | s -> Format.fprintf ppf "%s" s
    ;;

    let begin_paren () =
      meta_print_string "@[<1>("; print_seq ()
    ;;

    let end_paren () =
      print_seq ();
      meta_print_string "@,)@]"
    ;;

  end
;;

module Gen : Walk_spec.Make = Walk_gen.Gen;;

(* Generates a XML value printing program. *)
module Printer =
 functor (Io : Walk_spec.Out) ->
 Walker.Make (Gen) (Print_xml) (Io)
;;
